# *********************************************************************
# This Original Work is copyright of 51 Degrees Mobile Experts Limited.
# Copyright 2023 51 Degrees Mobile Experts Limited, Davidson House,
# Forbury Square, Reading, Berkshire, United Kingdom RG1 3EU.
#
# This Original Work is licensed under the European Union Public Licence
# (EUPL) v.1.2 and is subject to its terms as set out below.
#
# If a copy of the EUPL was not distributed with this file, You can obtain
# one at https://opensource.org/licenses/EUPL-1.2.
#
# The 'Compatible Licences' set out in the Appendix to the EUPL (as may be
# amended by the European Commission) shall be deemed incompatible for
# the purposes of the Work and the provisions of the compatibility
# clause in Article 5 of the EUPL shall not apply.
#
# If using the Work as, or as part of, a network application, by
# including the attribution notice(s) required under Article 5 of the EUPL
# in the end user terms of the application under an appropriate heading,
# such notice(s) shall fulfill the requirements of that article.
# ********************************************************************* */

from __future__ import absolute_import
from fiftyone_pipeline_engines.datafile import DataFile

import datetime

try:
    #python2
    from urllib import urlencode
except ImportError:
    #python3
    from urllib.parse import urlencode

class DeviceDetectionDataFile(DataFile):

    """!
    Extension of the DataFile class for DeviceDetection
    This helps construct the update url based on url parameters provided
    by the engine, provides methods to get the published date and
    update date of the datafile and refreshes the on premise 
    engine when the datafile is updated. 
    """

    def get_url_formatter(self):

        """!
        Generate a url for the data file update service to call
        These parameters are passed in to the constructor of the
        Device Detection On Premise engine
        @returns string : URL
        """

        query_params = {
            "Type": self.update_url_params["type"]
        }

        if ("license_keys" in self.update_url_params):
            query_params["licenseKeys"] = self.update_url_params["license_keys"]

        return self.update_url_params["base_url"] + '?' + urlencode(query_params)

    def swig_date_to_date(self, swig_date):

        """!
        Helper function to convert the SWIG wrapper date object
        into a python date
        @returns date
        """

        return datetime.datetime.strptime(str(swig_date.getYear()) + "-" + str(swig_date.getMonth()) + "-" + str(swig_date.getDay()), "%Y-%m-%d")
    
    def get_date_published(self):

        """!
        Get the date the datafile was published
        @returns date
        """
        
        if self.flow_element.engine:
            return self.swig_date_to_date(self.flow_element.engine.getPublishedTime())
        else:
            return datetime.datetime.now()

    def get_next_update(self):

        """!
        Get the date of the next datafile update
        @returns date
        """

        if self.flow_element.engine:
            return self.swig_date_to_date(self.flow_element.engine.getUpdateAvailableTime())
        else:
            return datetime.datetime.now()

    def refresh(self):

        """!
        Once the datafile has updated, refresh the engine
        """

        if not hasattr(self.flow_element, "engine"):
            self.flow_element.init_engine()
        
        if self.download == False:
            self.flow_element.engine.refreshDataFromMemory(self.data)
        else:
            self.flow_element.engine.refreshData()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _DeviceDetectionHashEngineModule
else:
    import _DeviceDetectionHashEngineModule

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_SwigPyIterator

    def value(self):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_value(self)

    def incr(self, n=1):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_decr(self, n)

    def distance(self, x):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_distance(self, x)

    def equal(self, x):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_equal(self, x)

    def copy(self):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_copy(self)

    def next(self):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_next(self)

    def __next__(self):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___next__(self)

    def previous(self):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_previous(self)

    def advance(self, n):
        return _DeviceDetectionHashEngineModule.SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        return _DeviceDetectionHashEngineModule.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.SwigPyIterator_swigregister(SwigPyIterator)

class MapStringStringSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___nonzero__(self)

    def __bool__(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___bool__(self)

    def __len__(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___len__(self)
    def __iter__(self):
        return self.key_iterator()
    def iterkeys(self):
        return self.key_iterator()
    def itervalues(self):
        return self.value_iterator()
    def iteritems(self):
        return self.iterator()

    def __getitem__(self, key):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___getitem__(self, key)

    def __delitem__(self, key):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___delitem__(self, key)

    def has_key(self, key):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_has_key(self, key)

    def keys(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_keys(self)

    def values(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_values(self)

    def items(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_items(self)

    def __contains__(self, key):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___contains__(self, key)

    def key_iterator(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_key_iterator(self)

    def value_iterator(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_value_iterator(self)

    def __setitem__(self, *args):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig___setitem__(self, *args)

    def asdict(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_asdict(self)

    def __init__(self, *args):
        _DeviceDetectionHashEngineModule.MapStringStringSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_MapStringStringSwig(*args))

    def empty(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_empty(self)

    def size(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_size(self)

    def swap(self, v):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_swap(self, v)

    def begin(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_begin(self)

    def end(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_end(self)

    def rbegin(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_rbegin(self)

    def rend(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_rend(self)

    def clear(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_clear(self)

    def get_allocator(self):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_get_allocator(self)

    def count(self, x):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_count(self, x)

    def erase(self, *args):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_erase(self, *args)

    def find(self, x):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_find(self, x)

    def lower_bound(self, x):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_lower_bound(self, x)

    def upper_bound(self, x):
        return _DeviceDetectionHashEngineModule.MapStringStringSwig_upper_bound(self, x)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_MapStringStringSwig

# Register MapStringStringSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.MapStringStringSwig_swigregister(MapStringStringSwig)

class VectorStringSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___nonzero__(self)

    def __bool__(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___bool__(self)

    def __len__(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___len__(self)

    def __getslice__(self, i, j):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___getslice__(self, i, j)

    def __setslice__(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___setslice__(self, *args)

    def __delslice__(self, i, j):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___delslice__(self, i, j)

    def __delitem__(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___delitem__(self, *args)

    def __getitem__(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___getitem__(self, *args)

    def __setitem__(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig___setitem__(self, *args)

    def pop(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_pop(self)

    def append(self, x):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_append(self, x)

    def empty(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_empty(self)

    def size(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_size(self)

    def swap(self, v):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_swap(self, v)

    def begin(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_begin(self)

    def end(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_end(self)

    def rbegin(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_rbegin(self)

    def rend(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_rend(self)

    def clear(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_clear(self)

    def get_allocator(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_get_allocator(self)

    def pop_back(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_pop_back(self)

    def erase(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_erase(self, *args)

    def __init__(self, *args):
        _DeviceDetectionHashEngineModule.VectorStringSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_VectorStringSwig(*args))

    def push_back(self, x):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_push_back(self, x)

    def front(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_front(self)

    def back(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_back(self)

    def assign(self, n, x):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_assign(self, n, x)

    def resize(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_resize(self, *args)

    def insert(self, *args):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_insert(self, *args)

    def reserve(self, n):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_reserve(self, n)

    def capacity(self):
        return _DeviceDetectionHashEngineModule.VectorStringSwig_capacity(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_VectorStringSwig

# Register VectorStringSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.VectorStringSwig_swigregister(VectorStringSwig)

class RequiredPropertiesConfigSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _DeviceDetectionHashEngineModule.RequiredPropertiesConfigSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_RequiredPropertiesConfigSwig(*args))
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_RequiredPropertiesConfigSwig

    def getProperties(self):
        return _DeviceDetectionHashEngineModule.RequiredPropertiesConfigSwig_getProperties(self)

# Register RequiredPropertiesConfigSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.RequiredPropertiesConfigSwig_swigregister(RequiredPropertiesConfigSwig)

class StringValueSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.StringValueSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_StringValueSwig())

    def hasValue(self):
        return _DeviceDetectionHashEngineModule.StringValueSwig_hasValue(self)

    def getNoValueMessage(self):
        return _DeviceDetectionHashEngineModule.StringValueSwig_getNoValueMessage(self)

    def getValue(self):
        return _DeviceDetectionHashEngineModule.StringValueSwig_getValue(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_StringValueSwig

# Register StringValueSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.StringValueSwig_swigregister(StringValueSwig)

class BoolValueSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.BoolValueSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_BoolValueSwig())

    def hasValue(self):
        return _DeviceDetectionHashEngineModule.BoolValueSwig_hasValue(self)

    def getNoValueMessage(self):
        return _DeviceDetectionHashEngineModule.BoolValueSwig_getNoValueMessage(self)

    def getValue(self):
        return _DeviceDetectionHashEngineModule.BoolValueSwig_getValue(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_BoolValueSwig

# Register BoolValueSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.BoolValueSwig_swigregister(BoolValueSwig)

class IntegerValueSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.IntegerValueSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_IntegerValueSwig())

    def hasValue(self):
        return _DeviceDetectionHashEngineModule.IntegerValueSwig_hasValue(self)

    def getNoValueMessage(self):
        return _DeviceDetectionHashEngineModule.IntegerValueSwig_getNoValueMessage(self)

    def getValue(self):
        return _DeviceDetectionHashEngineModule.IntegerValueSwig_getValue(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_IntegerValueSwig

# Register IntegerValueSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.IntegerValueSwig_swigregister(IntegerValueSwig)

class VectorStringValuesSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.VectorStringValuesSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_VectorStringValuesSwig())

    def hasValue(self):
        return _DeviceDetectionHashEngineModule.VectorStringValuesSwig_hasValue(self)

    def getNoValueMessage(self):
        return _DeviceDetectionHashEngineModule.VectorStringValuesSwig_getNoValueMessage(self)

    def getValue(self):
        return _DeviceDetectionHashEngineModule.VectorStringValuesSwig_getValue(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_VectorStringValuesSwig

# Register VectorStringValuesSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.VectorStringValuesSwig_swigregister(VectorStringValuesSwig)

class DoubleValueSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.DoubleValueSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_DoubleValueSwig())

    def hasValue(self):
        return _DeviceDetectionHashEngineModule.DoubleValueSwig_hasValue(self)

    def getNoValueMessage(self):
        return _DeviceDetectionHashEngineModule.DoubleValueSwig_getNoValueMessage(self)

    def getValue(self):
        return _DeviceDetectionHashEngineModule.DoubleValueSwig_getValue(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_DoubleValueSwig

# Register DoubleValueSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.DoubleValueSwig_swigregister(DoubleValueSwig)

class ResultsBaseSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ResultsBaseSwig

    def getAvailableProperties(self):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getAvailableProperties(self)

    def containsProperty(self, propertyName):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_containsProperty(self, propertyName)

    def getProperties(self):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getProperties(self)

    def getPropertyName(self, requiredPropertyIndex):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getPropertyName(self, requiredPropertyIndex)

    def getValues(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getValues(self, *args)

    def getValueAsString(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getValueAsString(self, *args)

    def getValueAsBool(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getValueAsBool(self, *args)

    def getValueAsInteger(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getValueAsInteger(self, *args)

    def getValueAsDouble(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsBaseSwig_getValueAsDouble(self, *args)

# Register ResultsBaseSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ResultsBaseSwig_swigregister(ResultsBaseSwig)

class Date(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def getYear(self):
        return _DeviceDetectionHashEngineModule.Date_getYear(self)

    def getMonth(self):
        return _DeviceDetectionHashEngineModule.Date_getMonth(self)

    def getDay(self):
        return _DeviceDetectionHashEngineModule.Date_getDay(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_Date

# Register Date in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.Date_swigregister(Date)

class PropertyMetaDataSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def getName(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getName(self)

    def getDataFilesWherePresent(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getDataFilesWherePresent(self)

    def getType(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getType(self)

    def getCategory(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getCategory(self)

    def getUrl(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getUrl(self)

    def getAvailable(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getAvailable(self)

    def getDisplayOrder(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getDisplayOrder(self)

    def getIsMandatory(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getIsMandatory(self)

    def getIsList(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getIsList(self)

    def getIsObsolete(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getIsObsolete(self)

    def getShow(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getShow(self)

    def getShowValues(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getShowValues(self)

    def getDescription(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataSwig_getDescription(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_PropertyMetaDataSwig

# Register PropertyMetaDataSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.PropertyMetaDataSwig_swigregister(PropertyMetaDataSwig)

class ComponentMetaDataSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def getComponentIdAsInt(self):
        return _DeviceDetectionHashEngineModule.ComponentMetaDataSwig_getComponentIdAsInt(self)

    def getComponentId(self):
        return _DeviceDetectionHashEngineModule.ComponentMetaDataSwig_getComponentId(self)

    def getName(self):
        return _DeviceDetectionHashEngineModule.ComponentMetaDataSwig_getName(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ComponentMetaDataSwig

# Register ComponentMetaDataSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ComponentMetaDataSwig_swigregister(ComponentMetaDataSwig)

class ProfileMetaDataSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def getProfileId(self):
        return _DeviceDetectionHashEngineModule.ProfileMetaDataSwig_getProfileId(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ProfileMetaDataSwig

# Register ProfileMetaDataSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ProfileMetaDataSwig_swigregister(ProfileMetaDataSwig)

class ValueMetaDataKeySwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, propertyName, valueName):
        _DeviceDetectionHashEngineModule.ValueMetaDataKeySwig_swiginit(self, _DeviceDetectionHashEngineModule.new_ValueMetaDataKeySwig(propertyName, valueName))

    def getPropertyName(self):
        return _DeviceDetectionHashEngineModule.ValueMetaDataKeySwig_getPropertyName(self)

    def getValueName(self):
        return _DeviceDetectionHashEngineModule.ValueMetaDataKeySwig_getValueName(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ValueMetaDataKeySwig

# Register ValueMetaDataKeySwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ValueMetaDataKeySwig_swigregister(ValueMetaDataKeySwig)

class ValueMetaDataSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def getName(self):
        return _DeviceDetectionHashEngineModule.ValueMetaDataSwig_getName(self)

    def getDescription(self):
        return _DeviceDetectionHashEngineModule.ValueMetaDataSwig_getDescription(self)

    def getUrl(self):
        return _DeviceDetectionHashEngineModule.ValueMetaDataSwig_getUrl(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ValueMetaDataSwig

# Register ValueMetaDataSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ValueMetaDataSwig_swigregister(ValueMetaDataSwig)

class ComponentMetaDataCollectionSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ComponentMetaDataCollectionSwig

    def getByKey(self, key):
        return _DeviceDetectionHashEngineModule.ComponentMetaDataCollectionSwig_getByKey(self, key)

    def getByIndex(self, index):
        return _DeviceDetectionHashEngineModule.ComponentMetaDataCollectionSwig_getByIndex(self, index)

    def getSize(self):
        return _DeviceDetectionHashEngineModule.ComponentMetaDataCollectionSwig_getSize(self)

# Register ComponentMetaDataCollectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ComponentMetaDataCollectionSwig_swigregister(ComponentMetaDataCollectionSwig)

class PropertyMetaDataCollectionSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_PropertyMetaDataCollectionSwig

    def getByKey(self, key):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataCollectionSwig_getByKey(self, key)

    def getByIndex(self, index):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataCollectionSwig_getByIndex(self, index)

    def getSize(self):
        return _DeviceDetectionHashEngineModule.PropertyMetaDataCollectionSwig_getSize(self)

# Register PropertyMetaDataCollectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.PropertyMetaDataCollectionSwig_swigregister(PropertyMetaDataCollectionSwig)

class ProfileMetaDataCollectionSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ProfileMetaDataCollectionSwig

    def getByKey(self, key):
        return _DeviceDetectionHashEngineModule.ProfileMetaDataCollectionSwig_getByKey(self, key)

    def getByIndex(self, index):
        return _DeviceDetectionHashEngineModule.ProfileMetaDataCollectionSwig_getByIndex(self, index)

    def getSize(self):
        return _DeviceDetectionHashEngineModule.ProfileMetaDataCollectionSwig_getSize(self)

# Register ProfileMetaDataCollectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ProfileMetaDataCollectionSwig_swigregister(ProfileMetaDataCollectionSwig)

class ValueMetaDataCollectionSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ValueMetaDataCollectionSwig

    def getByKey(self, key):
        return _DeviceDetectionHashEngineModule.ValueMetaDataCollectionSwig_getByKey(self, key)

    def getByIndex(self, index):
        return _DeviceDetectionHashEngineModule.ValueMetaDataCollectionSwig_getByIndex(self, index)

    def getSize(self):
        return _DeviceDetectionHashEngineModule.ValueMetaDataCollectionSwig_getSize(self)

# Register ValueMetaDataCollectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ValueMetaDataCollectionSwig_swigregister(ValueMetaDataCollectionSwig)

class MetaDataSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_MetaDataSwig

    def getComponents(self):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getComponents(self)

    def getProperties(self):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getProperties(self)

    def getProfiles(self):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getProfiles(self)

    def getValues(self):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getValues(self)

    def getValuesForProperty(self, property):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getValuesForProperty(self, property)

    def getValuesForProfile(self, profile):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getValuesForProfile(self, profile)

    def getComponentForProfile(self, profile):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getComponentForProfile(self, profile)

    def getComponentForProperty(self, property):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getComponentForProperty(self, property)

    def getDefaultProfileForComponent(self, component):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getDefaultProfileForComponent(self, component)

    def getDefaultValueForProperty(self, property):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getDefaultValueForProperty(self, property)

    def getPropertiesForComponent(self, component):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getPropertiesForComponent(self, component)

    def getPropertyForValue(self, value):
        return _DeviceDetectionHashEngineModule.MetaDataSwig_getPropertyForValue(self, value)

# Register MetaDataSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.MetaDataSwig_swigregister(MetaDataSwig)

class EngineBaseSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_EngineBaseSwig

    def setLicenseKey(self, licenceKey):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_setLicenseKey(self, licenceKey)

    def setDataUpdateUrl(self, updateUrl):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_setDataUpdateUrl(self, updateUrl)

    def getMetaData(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getMetaData(self)

    def getAutomaticUpdatesEnabled(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getAutomaticUpdatesEnabled(self)

    def processBase(self, evidence):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_processBase(self, evidence)

    def refreshData(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_refreshData(self)

    def refreshDataFromFile(self, fileName):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_refreshDataFromFile(self, fileName)

    def refreshDataFromMemory(self, data):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_refreshDataFromMemory(self, data)

    def getDataUpdateUrl(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getDataUpdateUrl(self)

    def getPublishedTime(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getPublishedTime(self)

    def getUpdateAvailableTime(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getUpdateAvailableTime(self)

    def getDataFilePath(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getDataFilePath(self)

    def getDataFileTempPath(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getDataFileTempPath(self)

    def getProduct(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getProduct(self)

    def getType(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getType(self)

    def getKeys(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getKeys(self)

    def getIsThreadSafe(self):
        return _DeviceDetectionHashEngineModule.EngineBaseSwig_getIsThreadSafe(self)

# Register EngineBaseSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.EngineBaseSwig_swigregister(EngineBaseSwig)

class ConfigBaseSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ConfigBaseSwig

    def setUseUpperPrefixHeaders(self, use):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_setUseUpperPrefixHeaders(self, use)

    def setUseTempFile(self, use):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_setUseTempFile(self, use)

    def setReuseTempFile(self, reuse):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_setReuseTempFile(self, reuse)

    def setTempDirectories(self, tempDirs):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_setTempDirectories(self, tempDirs)

    def getUseUpperPrefixHeaders(self):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_getUseUpperPrefixHeaders(self)

    def getUseTempFile(self):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_getUseTempFile(self)

    def getReuseTempFile(self):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_getReuseTempFile(self)

    def getTempDirectories(self):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_getTempDirectories(self)

    def getConcurrency(self):
        return _DeviceDetectionHashEngineModule.ConfigBaseSwig_getConcurrency(self)

# Register ConfigBaseSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ConfigBaseSwig_swigregister(ConfigBaseSwig)

class ConfigDeviceDetectionSwig(ConfigBaseSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def setUpdateMatchedUserAgent(self, update):
        return _DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_setUpdateMatchedUserAgent(self, update)

    def setMaxMatchedUserAgentLength(self, length):
        return _DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_setMaxMatchedUserAgentLength(self, length)

    def setAllowUnmatched(self, allow):
        return _DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_setAllowUnmatched(self, allow)

    def getUpdateMatchedUserAgent(self):
        return _DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_getUpdateMatchedUserAgent(self)

    def getMaxMatchedUserAgentLength(self):
        return _DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_getMaxMatchedUserAgentLength(self)

    def getAllowUnmatched(self):
        return _DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_getAllowUnmatched(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ConfigDeviceDetectionSwig

# Register ConfigDeviceDetectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ConfigDeviceDetectionSwig_swigregister(ConfigDeviceDetectionSwig)

class EvidenceBaseSwig(MapStringStringSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.EvidenceBaseSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_EvidenceBaseSwig())
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_EvidenceBaseSwig

# Register EvidenceBaseSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.EvidenceBaseSwig_swigregister(EvidenceBaseSwig)

class EvidenceDeviceDetectionSwig(EvidenceBaseSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.EvidenceDeviceDetectionSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_EvidenceDeviceDetectionSwig())
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_EvidenceDeviceDetectionSwig

# Register EvidenceDeviceDetectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.EvidenceDeviceDetectionSwig_swigregister(EvidenceDeviceDetectionSwig)

class EngineDeviceDetectionSwig(EngineBaseSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    defaultElementDataKey = property(_DeviceDetectionHashEngineModule.EngineDeviceDetectionSwig_defaultElementDataKey_get, _DeviceDetectionHashEngineModule.EngineDeviceDetectionSwig_defaultElementDataKey_set)

    def processDeviceDetection(self, *args):
        return _DeviceDetectionHashEngineModule.EngineDeviceDetectionSwig_processDeviceDetection(self, *args)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_EngineDeviceDetectionSwig

# Register EngineDeviceDetectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.EngineDeviceDetectionSwig_swigregister(EngineDeviceDetectionSwig)
cvar = _DeviceDetectionHashEngineModule.cvar

class ResultsDeviceDetectionSwig(ResultsBaseSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def getDeviceId(self):
        return _DeviceDetectionHashEngineModule.ResultsDeviceDetectionSwig_getDeviceId(self)

    def getUserAgent(self, resultIndex):
        return _DeviceDetectionHashEngineModule.ResultsDeviceDetectionSwig_getUserAgent(self, resultIndex)

    def getUserAgents(self):
        return _DeviceDetectionHashEngineModule.ResultsDeviceDetectionSwig_getUserAgents(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ResultsDeviceDetectionSwig

# Register ResultsDeviceDetectionSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ResultsDeviceDetectionSwig_swigregister(ResultsDeviceDetectionSwig)

class ResultsHashSwig(ResultsDeviceDetectionSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ResultsHashSwig

    def getDeviceId(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getDeviceId(self, *args)

    def getDifference(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getDifference(self, *args)

    def getMethod(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getMethod(self, *args)

    def getDrift(self, *args):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getDrift(self, *args)

    def getTrace(self):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getTrace(self)

    def getMatchedNodes(self):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getMatchedNodes(self)

    def getIterations(self):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getIterations(self)

    def getUserAgent(self, resultIndex):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getUserAgent(self, resultIndex)

    def getUserAgents(self):
        return _DeviceDetectionHashEngineModule.ResultsHashSwig_getUserAgents(self)

# Register ResultsHashSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ResultsHashSwig_swigregister(ResultsHashSwig)

class CollectionConfigSwig(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.CollectionConfigSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_CollectionConfigSwig())

    def setCapacity(self, capacity):
        return _DeviceDetectionHashEngineModule.CollectionConfigSwig_setCapacity(self, capacity)

    def setConcurrency(self, concurrency):
        return _DeviceDetectionHashEngineModule.CollectionConfigSwig_setConcurrency(self, concurrency)

    def setLoaded(self, loaded):
        return _DeviceDetectionHashEngineModule.CollectionConfigSwig_setLoaded(self, loaded)

    def getCapacity(self):
        return _DeviceDetectionHashEngineModule.CollectionConfigSwig_getCapacity(self)

    def getConcurrency(self):
        return _DeviceDetectionHashEngineModule.CollectionConfigSwig_getConcurrency(self)

    def getLoaded(self):
        return _DeviceDetectionHashEngineModule.CollectionConfigSwig_getLoaded(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_CollectionConfigSwig

# Register CollectionConfigSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.CollectionConfigSwig_swigregister(CollectionConfigSwig)

class ConfigHashSwig(ConfigDeviceDetectionSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _DeviceDetectionHashEngineModule.ConfigHashSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_ConfigHashSwig())

    def setHighPerformance(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setHighPerformance(self)

    def setBalanced(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setBalanced(self)

    def setBalancedTemp(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setBalancedTemp(self)

    def setLowMemory(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setLowMemory(self)

    def setMaxPerformance(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setMaxPerformance(self)

    def setDrift(self, drift):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setDrift(self, drift)

    def setDifference(self, difference):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setDifference(self, difference)

    def setConcurrency(self, concurrency):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setConcurrency(self, concurrency)

    def setUsePerformanceGraph(self, use):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setUsePerformanceGraph(self, use)

    def setUsePredictiveGraph(self, use):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setUsePredictiveGraph(self, use)

    def setTraceRoute(self, trace):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_setTraceRoute(self, trace)

    def getStrings(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getStrings(self)

    def getProperties(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getProperties(self)

    def getValues(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getValues(self)

    def getProfiles(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getProfiles(self)

    def getNodes(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getNodes(self)

    def getProfileOffsets(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getProfileOffsets(self)

    def getDrift(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getDrift(self)

    def getDifference(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getDifference(self)

    def getUsePerformanceGraph(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getUsePerformanceGraph(self)

    def getUsePredictiveGraph(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getUsePredictiveGraph(self)

    def getConcurrency(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getConcurrency(self)

    def getTraceRoute(self):
        return _DeviceDetectionHashEngineModule.ConfigHashSwig_getTraceRoute(self)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_ConfigHashSwig

# Register ConfigHashSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.ConfigHashSwig_swigregister(ConfigHashSwig)

class EngineHashSwig(EngineDeviceDetectionSwig):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _DeviceDetectionHashEngineModule.EngineHashSwig_swiginit(self, _DeviceDetectionHashEngineModule.new_EngineHashSwig(*args))

    def getPublishedTime(self):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_getPublishedTime(self)

    def getUpdateAvailableTime(self):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_getUpdateAvailableTime(self)

    def getDataFilePath(self):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_getDataFilePath(self)

    def getDataFileTempPath(self):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_getDataFileTempPath(self)

    def refreshData(self):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_refreshData(self)

    def refreshDataFromFile(self, fileName):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_refreshDataFromFile(self, fileName)

    def refreshDataFromMemory(self, data):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_refreshDataFromMemory(self, data)

    def process(self, *args):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_process(self, *args)

    def processBase(self, evidence):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_processBase(self, evidence)

    def processDeviceDetection(self, *args):
        return _DeviceDetectionHashEngineModule.EngineHashSwig_processDeviceDetection(self, *args)
    __swig_destroy__ = _DeviceDetectionHashEngineModule.delete_EngineHashSwig

# Register EngineHashSwig in _DeviceDetectionHashEngineModule:
_DeviceDetectionHashEngineModule.EngineHashSwig_swigregister(EngineHashSwig)






# *********************************************************************
# This Original Work is copyright of 51 Degrees Mobile Experts Limited.
# Copyright 2023 51 Degrees Mobile Experts Limited, Davidson House,
# Forbury Square, Reading, Berkshire, United Kingdom RG1 3EU.
#
# This Original Work is licensed under the European Union Public Licence
# (EUPL) v.1.2 and is subject to its terms as set out below.
#
# If a copy of the EUPL was not distributed with this file, You can obtain
# one at https://opensource.org/licenses/EUPL-1.2.
#
# The 'Compatible Licences' set out in the Appendix to the EUPL (as may be
# amended by the European Commission) shall be deemed incompatible for
# the purposes of the Work and the provisions of the compatibility
# clause in Article 5 of the EUPL shall not apply.
#
# If using the Work as, or as part of, a network application, by
# including the attribution notice(s) required under Article 5 of the EUPL
# in the end user terms of the application under an appropriate heading,
# such notice(s) shall fulfill the requirements of that article.
# ********************************************************************* 

from __future__ import absolute_import
from fiftyone_pipeline_core.pipelinebuilder import PipelineBuilder
from .devicedetection_onpremise import DeviceDetectionOnPremise
from fiftyone_devicedetection_shared.utils import merge_two_dicts

from fiftyone_pipeline_engines_fiftyone.share_usage import ShareUsage

class DeviceDetectionOnPremisePipelineBuilder(PipelineBuilder):
    """!
    The Device Detection Pipeline Builder allows you to easily
    Construct a pipeline containing the device detection cloud engine

    Internal function for getting evidence keys used by cloud engines
    
    """
    def __init__(self, settings = None, usage_sharing = True, **kwargs):

        """!

        Constructor for Device Detection Pipeline Builder
        Depending on parameters passed through, builds a pipeline
        with either the cloud or on premise device detection engines

        General pipeline settings:

        @type javascript_builder_settings: dict
        @param javascript_builder_settings: parameters for the pipeline's JavaScript builder to populate additional evidence from the clientside.
            - obj_name: the name of the client side object 
            with the JavaScript properties in it ('fod' by default)
            - protocol: The protocol ("http" or "https") used by 
            the client side callback url. This can be overridden 
            with header.protocol evidence
            - host The host of the client side callback url. 
            This can be overridden with header.host evidence.
            - endpoint The endpoint of the client side callback url
            * enable_cookies - whether cookies should be enabled
            * minify: Whether to minify the JavaScript
        @type cache: Cache
        @param cache: An instance of the fiftyone.pipeline.engines.cache
        class

        Cloud Engine Settings:

        @type resource_key: string
        @param resource_key: the 51Degrees cloud resource key

        On Premise Engine Settings

        @type usage_sharing : bool
        @param usage_sharing : Whether to enable usage sharing if using the on premise engine
        @type data_file_path: string
        @param data_file_path: path to the data file
        @type auto_update: bool
        @param auto_update: whether to automatically update the datafile when required
        @type restricted_properties: list
        @param restricted_properties: List of property keys to limit the engine to
        @type license_keys: string
        @param licence_keys: licencekeys to use for the data file update service
        @type download: bool
        @param download: whether to download the datafile or keep it in memory when it is returned from the datafile update service
        @type max_matched_useragents_length : int
        @param Number of characters to consider in the matched User-Agent. Ignored if update_matched_useragent is false
        @type update_matched_useragent: 
        @param: update_matched_useragent: True if the detection should record the matched characters from the target User-Agent
        @type drift: int
        @param drift: Set maximum drift in hash position to allow when processing HTTP headers
        @type difference: int
        @param difference: set the maximum difference to allow when processing HTTP headers. The difference is the difference in hash value between the hash that was found, and the hash that is being searched for. By default this is 0.
        @type allow_unmatched : bool
        @param allow_unmatched:  True if there should be at least one matched node in order for the results to be considered valid. By default, this is false
        @type performance_profile: string
        @param performance_profile: options are: LowMemory, MaxPerformance, Balanced, BalancedTemp, HighPerformance
        @type reuse_temp_file: bool
        @param reuse_temp_file: Indicates that an existing temp file may be used. This should be selected if multiple instances wish to use the same file to prevent high disk usage.
        @type concurrency: int
        @param concurrency: defaults to the number of cpus in the machine
        @type update_on_start : bool
        @param update_on_start : When this is set to true the datafile is updated / downloaded immediately on initialisation. This is useful if no initial datafile is present.
        @type file_system_watcher: bool
        @param file_system_watcher: whether to check the datafile's path for changes and update the connected FlowElement's data 
        @type polling_interval: int
        @param polling_interval: How often to poll for updates to the datafile (minutes)
        @type update_time_maximum_randomisation : int
        @param update_time_maximum_randomisation :
        Maximum randomisation offset in seconds to polling time interval
        @type verify_md5 : bool
        @type create_temp_copy: bool
        @param create_temp_copy: whether to copy datafile to temporary location when updating
        @type data_file_update_base_url: string
        @param data_file_update_base_url: base url for the datafile update service
        @param data_file_update_service: service for the pipeline to use for auto updates instead of the default
        @type data_file_update_service: DataFileUpdateService
        """
        if settings is None:
            settings = {}
            
        settings = merge_two_dicts(dict(**kwargs), locals())

        settings = merge_two_dicts(settings, settings["settings"])

        del settings["self"]

        super(DeviceDetectionOnPremisePipelineBuilder, self).__init__(settings)

        # Add specific engines


        del settings["settings"]
        device = DeviceDetectionOnPremise(**settings)
        
        # If usage sharing enabled, add usage sharing engine

        if usage_sharing:
            self.add(ShareUsage())

        if "cache" in settings:
            device.set_cache(settings["cache"])
        
        self.add(device)


# *********************************************************************
# This Original Work is copyright of 51 Degrees Mobile Experts Limited.
# Copyright 2023 51 Degrees Mobile Experts Limited, Davidson House,
# Forbury Square, Reading, Berkshire, United Kingdom RG1 3EU.
#
# This Original Work is licensed under the European Union Public Licence
# (EUPL) v.1.2 and is subject to its terms as set out below.
#
# If a copy of the EUPL was not distributed with this file, You can obtain
# one at https://opensource.org/licenses/EUPL-1.2.
#
# The 'Compatible Licences' set out in the Appendix to the EUPL (as may be
# amended by the European Commission) shall be deemed incompatible for
# the purposes of the Work and the provisions of the compatibility
# clause in Article 5 of the EUPL shall not apply.
#
# If using the Work as, or as part of, a network application, by
# including the attribution notice(s) required under Article 5 of the EUPL
# in the end user terms of the application under an appropriate heading,
# such notice(s) shall fulfill the requirements of that article.
# ********************************************************************* 
DEVICE_ID_DESCRIPTION = ('Consists of four components separated by a hyphen '
                         'symbol: Hardware-Platform-Browser-IsCrawler where '
                         'each Component represents an ID of the corresponding '
                         'Profile.')
USER_AGENTS_DESCRIPTION = 'The matched User-Agents.'
DIFFERENCE_DESCRIPTION = ('Used when detection method is not Exact or None. '
                          'This is an integer value and the larger the value '
                          'the less confident the detector is in this result.')
MATCHED_NODES_DESCRIPTION = ('Indicates the number of hash nodes matched '
                             'within the evidence.')
DRIFT_DESCRIPTION = ('Total difference in character positions where the '
                     'substrings hashes were found away from where they '
                     'were expected.')
ITERATIONS_DESCRIPTION = ('The number of iterations carried out in order to '
                          'find a match. This is the number of nodes in the '
                          'graph which have been visited.')
METHOD_DESCRIPTION = 'The method used to determine the match result.'